with ada.text_io; use ada.text_io;
with ada.float_text_io; use ada.float_text_io;
with ada.strings.fixed; use ada.strings.fixed;
with ada.strings; use ada.strings;

PACKAGE BODY Fctbivqualquant IS
	
   	dEBUG_MODE : constant boolean := false;
	procedure debug(debug_message : in string) is 
	begin
		if dEBUG_MODE then	
			put_line("Debug : " & debug_message);
		end if;
	end debug;


   ----------------------------------- Dcoupage des sous-populations  ----------------------------------------------------

   FUNCTION DecoupageSousPopulations (
         MonAnalyseQualiteQuantite : ANA_QUAL_X_QUANT)
     RETURN T_ANA_QUANT IS
      TabRetour  : T_ANA_QUANT;
      Pos        : Integer;
      QualiteTMP : String (1 .. 50);

   BEGIN
      FOR I IN 1..MonAnalyseQualiteQuantite.Qual.Nombre_Etats_Possibles LOOP
         TabRetour(I).Quant := MonAnalyseQualiteQuantite.Quant;
         TabRetour(I).Nb_Elements := 0;
      END LOOP;
      FOR I IN 1..MonAnalyseQualiteQuantite.Nb_Elements LOOP
         QualiteTMP := MonAnalyseQualiteQuantite.Donnees(I).Val_Qual;
         FOR J IN 1..
               MonAnalyseQualiteQuantite.Qual.Nombre_Etats_Possibles LOOP
            IF(QualiteTMP = MonAnalyseQualiteQuantite.Qual.Etats_Possibles(
                     J)) THEN
               Pos:=J;
            END IF;
         END LOOP;
         TabRetour(Pos).Nb_Elements := TabRetour(Pos).Nb_Elements+1;
         TabRetour(Pos).Donnees(TabRetour(Pos).Nb_Elements) :=
            MonAnalyseQualiteQuantite.Donnees(I).Val_Quant;
      END LOOP;
      RETURN TabRetour;
   END DecoupageSousPopulations;

   ----------------------------------- Analyses univaries sur les sous populations -----------------------------------

   FUNCTION AnalyseUnivarieeSousPopulations (
         Tab              : T_ANA_QUANT;
         NbEtatsPossibles : Integer)
     RETURN T_RES_ANA_QUANT IS
      TabRes : T_RES_ANA_QUANT;

   BEGIN
      FOR I IN 1..NbEtatsPossibles LOOP
         TabRes(I) := TraitementQuantitatifUnivarie(Tab(I));
      END LOOP;
      RETURN TabRes;
   END AnalyseUnivarieeSousPopulations;

   ----------------------------------- Calcul de la Moyenne totale ----------------------------------------------------

   FUNCTION CalculerMoyenneTotale (
         MonAnalyseQualiteQuantite : ANA_QUAL_X_QUANT)
     RETURN Float IS
      Moyenne : Float;

   BEGIN
      Moyenne :=0.0;
      FOR I IN 1..MonAnalyseQualiteQuantite.Nb_Elements LOOP
         Moyenne := Moyenne + MonAnalyseQualiteQuantite.Donnees(I).
            Val_Quant;
      END LOOP;
      RETURN (Moyenne/Float(MonanalyseQualiteQuantite.Nb_Elements));
   END CalculerMoyenneTotale;

   ----------------------------------- Calcul de la variance Totale ----------------------------------------------------

   FUNCTION CalculerVarianceTotale (
         MonAnalyseQualiteQuantite : ANA_QUAL_X_QUANT;
         MoyenneTotale             : Float)
     RETURN Float IS
      Variance : Float;

   BEGIN
      Variance:=0.0;
      FOR I IN 1..MonAnalyseQualiteQuantite.Nb_Elements LOOP
         Variance := Variance + (MonAnalyseQualiteQuantite.Donnees(I).
            Val_Quant - MoyenneTotale)**2;
      END LOOP;
      RETURN (Variance/Float(MonAnalyseQualiteQuantite.Nb_Elements));
   END CalculerVarianceTotale;



   ----------------------------------- Calcul de la variance inter -------------------------------------------------------

   FUNCTION CalculerVarianceInter (
         Tab           : T_RES_ANA_QUANT;
         NbElementsTab : Integer;
         MoyenneTotale : Float;
         Card          : Integer)
     RETURN Float IS
      VarianceTotale : Float;

   BEGIN
      VarianceTotale := 0.0;
      FOR I IN 1..NbElementsTab LOOP
         VarianceTotale := VarianceTotale + Float(Tab(I).Nb_Elements)*(
            Tab(I).Moyenne - MoyenneTotale)**2;
      END LOOP;
      RETURN VarianceTotale/Float(Card);
   END CalculerVarianceInter;

   ----------------------------------- Calcul de la variance intra ----------------------------------------------------


   FUNCTION CalculerVarianceIntra (
         Tab           : T_RES_ANA_QUANT;
         NbElementsTab : Integer;
         Card          : Integer)
     RETURN Float IS
      VarianceIntra : Float;

   BEGIN
      VarianceIntra := 0.0;
      FOR I IN 1..NbElementsTab LOOP
         VarianceIntra := VarianceIntra + (Tab(I).Variance*Float(Tab(I).
               Nb_Elements));
      END LOOP;
      RETURN VarianceIntra/Float(Card);
   END CalculerVarianceIntra;


   ----------------------------------- Calcul du lien ----------------------------------------------------------------

   FUNCTION CalculerLien (
         VarianceTotale : Float;
         VarianceInter  : Float)
     RETURN Float IS
   BEGIN
      RETURN VarianceInter/VarianceTotale;
   END CalculerLien;

   ---------------------------------------------------------------------
   -----                     PARTIE SUPPLEMENTAIRE                ------
   ---------------------------------------------------------------------
	procedure afficher(analyse : in ANA_QUAL_X_QUANT) is
	begin
		put_line(" Qualite     : " & analyse.qual.Nom_qualite);
		put_line(" Quantite    : " & analyse.quant.Nom_quantite);
		put_line(" nb _elements : " & integer'image(analyse.nb_elements));

		put(" [ ");
		for J in 1..analyse.nb_elements loop
			put("(");
			put( trim(analyse.donnees(j).val_qual,both) );
			put(", ");
			put( analyse.donnees(j).val_quant, 2, 2, 0);
			put(")");
			if J /= analyse.nb_elements then
				put(", ");
			end if;
		end loop;
		put_line("]");
	end afficher;

   procedure addElement(analyse : in out ANA_QUAL_X_QUANT; el_qual : in string; el_quant : in Float) is
   		doublet : DOUBLET_QUAL_QUANT;
	begin
		debug(" value Qual  : '" & trim(el_qual,both) & "'");
		debug(" value Qual range : '" & integer'image(el_qual'first) & ".." & integer'image(el_qual'last) & "'");
		debug(" value Quant : '" & float'image(el_quant) & "'");
		debug(" value doublet Qual  : '" & doublet.val_qual & "'");
		debug(" value doublet Quant : '" & float'image(doublet.val_quant) & "'");
		doublet.val_qual(el_qual'range) := el_qual;
		doublet.val_quant := el_quant;

		analyse.Nb_Elements := analyse.Nb_Elements + 1;
		analyse.donnees( analyse.Nb_Elements ) := doublet;
	end addElement;
	 
   procedure get_caracteres(
   		etudiants : in liste_etudiant;         --les etudiants
		choix1 : in mode_qualitatif;           --le caractere 1 choisi
		choix2 : in mode_quantitatif;          --le caractere 2 choisi
		analyse : out ANA_QUAL_X_QUANT         --structure a remplir
		) is
		aux_etu : liste_etudiant;
		aux_promo : liste_promo;
		aux_module : liste_module;

		etudiant_tmp : etudiant;
		Promo_tmp: promo;
		module_tmp : module;

		value_qual  : String(1..100) := (others => ' ');
		value_quant : float := -1.0;
	begin
		--Creation struct qualite
		Set_qualite_name(analyse.qual,choix1);
		--Creation struct qualite
		Set_quantite_name(analyse.quant,choix2);

		debug("structure qualite.qual1 cre : '" & trim(analyse.qual.Nom_qualite,both) & "'");
		debug("structure qualite.qual2 cre : '" & trim(analyse.quant.Nom_quantite,both) & "'");

		--Creation struct analyse qualite
		aux_etu := etudiants;
		while aux_etu /= null loop
			get_next(aux_etu,etudiant_tmp);

			if 
				choix1 = Genre or 
				Choix1 = option or
				Choix1 = diplome or
				Choix1 = diplome_option or
				Choix1 = origine_departement
				then
					declare
						value_tmp : string := extract_value(etudiant_tmp,choix1);
					begin
						value_qual(value_tmp'range) := value_tmp;
					end;
			end if;

			if choix2 = age then
				value_quant := extract_value(etudiant_tmp,choix2);
			end if;

			--TEST SI LES DEUX VALEURS SONT REMPLIS

			aux_promo := etudiant_tmp.promos;
			while aux_promo /= null loop
				get_next(aux_promo,promo_tmp);

				if
					choix1 = nom_promo or 
					Choix1 = statut_promo
					then
						declare
							value_tmp : string := extract_value(promo_tmp,choix1);
						begin
							value_qual(value_tmp'range) := value_tmp;
						end;
				end if;

				--TEST SI LES DEUX VALEURS SONT REMPLIS

				aux_module := promo_tmp.matieres;
				while aux_module /= null loop
					get_next(aux_module,module_tmp);
						
						if choix1 = matiere then
							declare
								value_tmp : string := extract_value(module_tmp,choix1);
							begin
								value_qual(value_tmp'range) := value_tmp;
							end;
						end if;

						if 
							choix2 = note_math or
							choix2 = note_dge or
							choix2 = note_info or
							choix2 = note_phy_eln or
							choix2 = note_rh or
							choix2 = note_varia or
							choix2 = note_option or
							choix2 = note_all
							then
								value_quant := extract_value(module_tmp,choix2);
						end if;

					    debug("Etudiant " & integer'image(etudiant_tmp.id));
					    debug("   -> " & mode_qualitatif'image(choix1) & " : '" & trim(value_qual,both) & "'");
					    debug("   -> " & mode_quantitatif'image(choix2) & " : '" & float'image(value_quant) & "'");
			
						if ( value_quant /= -1.0 ) then
							addElement(analyse,trim(value_qual,both),value_quant);
						else
							debug("non ajout !");
						end if;
						
						if choix1 = matiere then
							value_qual := (others => ' ');
						end if;

				end loop; --loop matiere
			end loop; --loop promo
			value_qual := (others => ' ');
		end loop;--loop  etudiant

		debug("fin remplissage structure");
		if debug_mode then
			new_line;
			afficher(analyse);
			new_line;
		end if;

	end get_caracteres;

END Fctbivqualquant;


