WITH Const;
USE Const;
with ada.text_io; use ada.text_io;
with etude; use etude;

PACKAGE FctUniQuant IS

   TYPE T_REELS IS ARRAY (1 .. Max_Ele) OF Float;
   ----- On dfinie la structure quantit
   TYPE QUANTITE IS
      RECORD
         Nom_Quantite : String (1 .. 50) := (others => ' ');
         Valeur_Min   : float := float'last;
         Valeur_Max   : float := float'first;
      END RECORD;
   -------------------------------------

   ----- On dfinie la structure ANA_QUANT
   TYPE ANA_QUANT IS
      RECORD
         Quant       : QUANTITE;
         Nb_Elements : Integer RANGE 0 .. Max_Ele := 0;
         Donnees     : T_REELS := (others => 0.0);
      END RECORD;
   -------------------------------------



   ----- On dfinie la structure RES_ANA_QUANT
   TYPE RES_ANA_QUANT IS
      RECORD
         Quant              : QUANTITE;
         Nb_Elements        : Integer RANGE 0 .. Max_Ele := 0;
         Moyenne            : FLoat := 0.0;
         Mediane            : Float := 0.0;
         Variance           : Float := 0.0;
         Ecart_Type         : Float := 0.0;
         Min                : Float := 0.0;
         Max                : Float := 0.0;
         Premier_Quartile   : Float := 0.0;
         Troisieme_Quartile : Float := 0.0;
      END RECORD;
   -------------------------------------

   PROCEDURE TrierTableauCroissant (
         MaQuantite : IN OUT ANA_QUANT);
   FUNCTION CalculerMoyenne (
         MaQuantite : IN     ANA_QUANT)
     RETURN Float;
   FUNCTION CalculerMin (
         MaQuantite : IN     ANA_QUANT)
     RETURN Float;
   FUNCTION CalculerMax (
         MaQuantite : IN     ANA_QUANT)
     RETURN Float;
   FUNCTION CalculerVariance (
         MaQuantite : IN     ANA_QUANT;
         Moyenne    : IN     Float)
     RETURN Float;
   FUNCTION CalculerEcartType (
         MaVariance : Float)
     RETURN Float;
   PROCEDURE CalculerQuantiles (
         MaQuantite : IN     ANA_QUANT;
         Mediane    : IN OUT Float;
         Quartile1  : IN OUT Float;
         Quartile3  : IN OUT Float);
   PROCEDURE Afficher (
         Mon_Resultat : IN     RES_ANA_QUANT);
   FUNCTION TraitementQuantitatifUnivarie (
         MAQ : IN     ANA_QUANT)
     RETURN RES_ANA_QUANT;

	 ----------------------------------------------------
	 ---NOUVELLE PARTIE POUR RELIER ETUDIANT-CARACTERE---
	 ----------------------------------------------------
	 
	 --Ajoute un donne dans la structure analytique et mets  jour les variables adjacentes
	procedure addElement(analyse_quantite : in out ana_quant; element : in float);

	--Creation d'un type qui enumere les modes quantitatif
	type Mode_Quantitatif is (
   		note_math,
		note_dge,
		note_info,
		note_phy_eln,
		note_rh,
		note_varia,
		note_option,
		note_all,
		age
		);

		--Package pour le IO sur le type quantitatif
   package Mode_Quant_io is new Enumeration_io(mode_quantitatif); use Mode_quant_io;
   
   --Fixe le nom d'une structure quantit grce au type pass en paramtre
   procedure Set_quantite_name(quant : in out quantite;choix : in mode_quantitatif);

   --Extraite la valeur du mode pass en argument dans une structure etudiante
   function extract_value(student : in etudiant; choix : Mode_Quantitatif) return float;
   --Extraite la valeur du mode pass en argument dans une structure module
   function extract_value(module_tmp : in module; choix : Mode_Quantitatif) return float;
   
   --Recupere toutes les donnes correspondant au caractere chosii dans les etudiants et les mets dans la structure analytique
   procedure get_caractere(
   		etudiants : in liste_etudiant;         --les etudiants
		choix : in mode_quantitatif;           --le caractere choisi
		analyse : out ana_quant);              --structure a remplir

END FctUniQuant;

